@@ -139,7 +139,9 @@ class Query(LimitedParamBenchmark):
         self.T.query(self.queries, p=p)
 
     # Retain old benchmark results (remove this if changing the benchmark)
-    time_query.version = "327bc0627d5387347e9cdcf4c52a550c813bb80a859eeb0f3e5bfe6650a8a1db"
+    time_query.version = (
+        "327bc0627d5387347e9cdcf4c52a550c813bb80a859eeb0f3e5bfe6650a8a1db"
+    )
 
 
 class Radius(LimitedParamBenchmark):
@@ -184,8 +186,12 @@ class Radius(LimitedParamBenchmark):
         self.T.query_pairs(probe_radius, p=p)
 
     # Retain old benchmark results (remove this if changing the benchmark)
-    time_query_ball_point.version = "e0c2074b35db7e5fca01a43b0fba8ab33a15ed73d8573871ea6feb57b3df4168"
-    time_query_pairs.version = "cf669f7d619e81e4a09b28bb3fceaefbdd316d30faf01524ab33d41661a53f56"
+    time_query_ball_point.version = (
+        "e0c2074b35db7e5fca01a43b0fba8ab33a15ed73d8573871ea6feb57b3df4168"
+    )
+    time_query_pairs.version = (
+        "cf669f7d619e81e4a09b28bb3fceaefbdd316d30faf01524ab33d41661a53f56"
+    )
 
 
 class Neighbors(LimitedParamBenchmark):
@@ -202,7 +208,8 @@ class Neighbors(LimitedParamBenchmark):
     num_param_combinations = 17
 
     def setup(self, mn1n2, p, probe_radius, boxsize, leafsize, cls):
-        LimitedParamBenchmark.setup(self, mn1n2, p, probe_radius, boxsize, leafsize, cls)
+        LimitedParamBenchmark.setup(self, mn1n2, p, probe_radius,
+                                    boxsize, leafsize, cls)
 
         m, n1, n2 = mn1n2
 
@@ -215,7 +222,8 @@ class Neighbors(LimitedParamBenchmark):
         self.T1 = cKDTree(self.data1, boxsize=boxsize, leafsize=leafsize)
         self.T2 = cKDTree(self.data2, boxsize=boxsize, leafsize=leafsize)
 
-    def time_sparse_distance_matrix(self, mn1n2, p, probe_radius, boxsize, leafsize, cls):
+    def time_sparse_distance_matrix(self, mn1n2, p, probe_radius,
+                                    boxsize, leafsize, cls):
         self.T1.sparse_distance_matrix(self.T2, probe_radius, p=p)
 
     def time_count_neighbors(self, mn1n2, p, probe_radius, boxsize, leafsize, cls):
@@ -227,11 +235,16 @@ class Neighbors(LimitedParamBenchmark):
         if cls != 'cKDTree_weighted':
             self.T1.count_neighbors(self.T2, probe_radius, p=p)
         else:
-            self.T1.count_neighbors(self.T2, probe_radius, weights=(self.w1, self.w2), p=p)
+            self.T1.count_neighbors(self.T2, probe_radius,
+                                    weights=(self.w1, self.w2), p=p)
 
     # Retain old benchmark results (remove this if changing the benchmark)
-    time_sparse_distance_matrix.version = "9aa921dce6da78394ab29d949be27953484613dcf9c9632c01ae3973d4b29596"
-    time_count_neighbors.version = "830287f1cf51fa6ba21854a60b03b2a6c70b2f2485c3cdcfb19a360e0a7e2ca2"
+    time_sparse_distance_matrix.version = (
+        "9aa921dce6da78394ab29d949be27953484613dcf9c9632c01ae3973d4b29596"
+    )
+    time_count_neighbors.version = (
+        "830287f1cf51fa6ba21854a60b03b2a6c70b2f2485c3cdcfb19a360e0a7e2ca2"
+    )
 
 
 class CNeighbors(Benchmark):
@@ -281,6 +294,17 @@ def generate_spherical_points(num_points):
     points /= np.linalg.norm(points, axis=1)[:, np.newaxis]
     return points
 
+
+def generate_circle_points(num_points):
+    # try to avoid full circle degeneracy
+    # at 2 * pi
+    angles = np.linspace(0, 1.9999 * np.pi, num_points)
+    points = np.empty(shape=(num_points, 2))
+    points[..., 0] = np.cos(angles)
+    points[..., 1] = np.sin(angles)
+    return points
+
+
 class SphericalVor(Benchmark):
     params = [10, 100, 1000, 5000, 10000]
     param_names = ['num_points']
@@ -311,15 +335,21 @@ class SphericalVorSort(Benchmark):
 
 
 class SphericalVorAreas(Benchmark):
-    params = [10, 100, 1000, 5000, 10000]
-    param_names = ['num_points']
+    params = ([10, 100, 1000, 5000, 10000],
+              [2, 3])
+    param_names = ['num_points', 'ndim']
 
-    def setup(self, num_points):
-        self.points = generate_spherical_points(num_points)
+    def setup(self, num_points, ndim):
+        if ndim == 2:
+            center = np.zeros(2)
+            self.points = generate_circle_points(num_points)
+        else:
+            center = np.zeros(3)
+            self.points = generate_spherical_points(num_points)
         self.sv = SphericalVoronoi(self.points, radius=1,
-                                   center=np.zeros(3))
+                                   center=center)
 
-    def time_spherical_polygon_area_calculation(self, num_points):
+    def time_spherical_polygon_area_calculation(self, num_points, ndim):
         """Time the area calculation in the Spherical Voronoi code."""
         self.sv.calculate_areas()
 
@@ -331,7 +361,7 @@ class Xdist(Benchmark):
                'hamming', 'jaccard', 'jensenshannon', 'chebyshev', 'canberra',
                'braycurtis', 'mahalanobis', 'yule', 'dice', 'kulczynski1',
                'rogerstanimoto', 'russellrao', 'sokalmichener', 'sokalsneath',
-               'wminkowski', 'minkowski-P3'])
+               'minkowski-P3'])
     param_names = ['num_points', 'metric']
 
     def setup(self, num_points, metric):
@@ -342,9 +372,6 @@ class Xdist(Benchmark):
             # p=2 is just the euclidean metric, try another p value as well
             self.kwargs = {'p': 3.0}
             self.metric = 'minkowski'
-        elif metric == 'wminkowski':
-            # use an equal weight vector since weights are required
-            self.kwargs = {'w': np.ones(3)}
         else:
             self.kwargs = {}
 
