@@ -18,7 +18,7 @@ Type codes used:
 See sparsetools.cxx for more details.
 
 """
-import optparse
+import argparse
 import os
 from stat import ST_MTIME
 
@@ -352,12 +352,13 @@ def parse_routine(name, args, types):
 
 
 def main():
-    p = optparse.OptionParser(usage=(__doc__ or '').strip())
-    p.add_option("--no-force", action="store_false",
-                 dest="force", default=True)
-    p.add_option("-o", "--outdir", type=str,
-                 help="Relative path to the output directory")
-    options, args = p.parse_args()
+    p = argparse.ArgumentParser(usage=(__doc__ or '').strip())
+
+    p.add_argument("--no-force", action="store_false",
+                   dest="force", default=True)
+    p.add_argument("-o", "--outdir", type=str,
+                   help="Relative path to the output directory")
+    options = p.parse_args()
 
     names = []
 
@@ -396,9 +397,6 @@ def main():
         if options.outdir:
             # Used by Meson (options.outdir == scipy/sparse/sparsetools)
             outdir = os.path.join(os.getcwd(), options.outdir)
-        else:
-            # Used by setup.py
-            outdir = os.path.join(os.path.dirname(__file__), 'sparsetools')
 
         dst = os.path.join(outdir,
                            unit_name + '_impl.h')
@@ -421,12 +419,15 @@ def main():
     # Generate code for method struct
     method_defs = ""
     for name in names:
-        method_defs += f"NPY_VISIBILITY_HIDDEN PyObject *{name}_method(PyObject *, PyObject *);\n"
+        method_defs += (f"NPY_VISIBILITY_HIDDEN PyObject *{name}"
+                        f"_method(PyObject *, PyObject *);\n")
 
     method_struct = """\nstatic struct PyMethodDef sparsetools_methods[] = {"""
     for name in names:
-        method_struct += """
-        {"%(name)s", (PyCFunction)%(name)s_method, METH_VARARGS, NULL},""" % dict(name=name)
+        method_struct += ("""
+            {{"{name}", (PyCFunction){name}_method, METH_VARARGS, NULL}},"""
+            .format(**dict(name=name))
+        )
     method_struct += """
         {NULL, NULL, 0, NULL}
     };"""
