@@ -4,16 +4,14 @@
 import itertools
 import platform
 import sys
-import numpy as np
-
-from numpy.testing import (assert_array_equal, assert_allclose,
-                           suppress_warnings)
 import pytest
 
-
+import numpy as np
+from numpy.testing import assert_array_equal, assert_allclose
 from numpy import zeros, arange, array, ones, eye, iscomplexobj
-from scipy.linalg import norm
-from scipy.sparse import spdiags, csr_matrix, SparseEfficiencyWarning, kronsum
+from numpy.linalg import norm
+
+from scipy.sparse import spdiags, csr_matrix, kronsum
 
 from scipy.sparse.linalg import LinearOperator, aslinearoperator
 from scipy.sparse.linalg._isolve import (bicg, bicgstab, cg, cgs,
@@ -27,6 +25,13 @@ from scipy.sparse.linalg._isolve import (bicg, bicgstab, cg, cgs,
 # list of all solvers under test
 _SOLVERS = [bicg, bicgstab, cg, cgs, gcrotmk, gmres, lgmres,
             minres, qmr, tfqmr]
+
+pytestmark = [
+    # remove this once atol defaults to 0.0 for all methods
+    pytest.mark.filterwarnings("ignore:.*called without specifying.*"),
+]
+
+
 # create parametrized fixture for easy reuse in tests
 @pytest.fixture(params=_SOLVERS, scope="session")
 def solver(request):
@@ -53,6 +58,15 @@ class Case:
         else:
             self.nonconvergence = nonconvergence
 
+
+class SingleTest:
+    def __init__(self, A, b, solver, casename, convergence=True):
+        self.A = A
+        self.b = b
+        self.solver = solver
+        self.name = casename + '-' + solver.__name__
+        self.convergence = convergence
+
     def __repr__(self):
         return f"<{self.name}>"
 
@@ -184,9 +198,25 @@ class IterativeParams:
                                )
                           )
 
+    def generate_tests(self):
+        # generate test cases with skips applied
+        tests = []
+        for case in self.cases:
+            for solver in _SOLVERS:
+                if (solver in case.skip):
+                    continue
+                if solver in case.nonconvergence:
+                    tests += [SingleTest(case.A, case.b, solver, case.name,
+                                         convergence=False)]
+                else:
+                    tests += [SingleTest(case.A, case.b, solver, case.name)]
+        return tests
 
-cases = IterativeParams().cases
-@pytest.fixture(params=cases, ids=[x.name for x in cases], scope="session")
+
+cases = IterativeParams().generate_tests()
+
+
+@pytest.fixture(params=cases, ids=[x.name for x in cases], scope="module")
 def case(request):
     """
     Fixture for all cases in IterativeParams
@@ -194,9 +224,11 @@ def case(request):
     return request.param
 
 
-def check_maxiter(solver, case):
+def test_maxiter(case):
+    if not case.convergence:
+        pytest.skip("Solver - Breakdown case, see gh-8829")
     A = case.A
-    tol = 1e-12
+    rtol = 1e-12
 
     b = case.b
     x0 = 0 * b
@@ -206,58 +238,39 @@ def check_maxiter(solver, case):
     def callback(x):
         residuals.append(norm(b - case.A * x))
 
-    x, info = solver(A, b, x0=x0, tol=tol, maxiter=1, callback=callback)
+    x, info = case.solver(A, b, x0=x0, rtol=rtol, maxiter=1, callback=callback)
 
     assert len(residuals) == 1
     assert info == 1
 
 
-def test_maxiter(solver, case):
-    if solver in case.skip + case.nonconvergence:
-        pytest.skip("unsupported combination")
-    with suppress_warnings() as sup:
-        sup.filter(DeprecationWarning, ".*called without specifying.*")
-        check_maxiter(solver, case)
-
-
-def assert_normclose(a, b, tol=1e-8):
-    residual = norm(a - b)
-    tolerance = tol * norm(b)
-    assert residual < tolerance
-
-
-def check_convergence(solver, case):
+def test_convergence(case):
     A = case.A
 
     if A.dtype.char in "dD":
-        tol = 1e-8
+        rtol = 1e-8
     else:
-        tol = 1e-2
+        rtol = 1e-2
 
     b = case.b
     x0 = 0 * b
 
-    x, info = solver(A, b, x0=x0, tol=tol)
+    x, info = case.solver(A, b, x0=x0, rtol=rtol)
 
     assert_array_equal(x0, 0 * b)  # ensure that x0 is not overwritten
-    if solver not in case.nonconvergence:
+    if case.convergence:
         assert info == 0
-        assert_normclose(A @ x, b, tol=tol)
+        assert norm(A @ x - b) <= norm(b) * rtol
     else:
         assert info != 0
-        assert np.linalg.norm(A @ x - b) <= np.linalg.norm(b)
+        assert norm(A @ x - b) <= norm(b)
 
 
-def test_convergence(solver, case):
-    if solver in case.skip:
-        pytest.skip("unsupported combination")
-    with suppress_warnings() as sup:
-        sup.filter(DeprecationWarning, ".*called without specifying.*")
-        check_convergence(solver, case)
+def test_precond_dummy(case):
+    if not case.convergence:
+        pytest.skip("Solver - Breakdown case, see gh-8829")
 
-
-def check_precond_dummy(solver, case):
-    tol = 1e-8
+    rtol = 1e-8
 
     def identity(b, which=None):
         """trivial preconditioner"""
@@ -277,129 +290,104 @@ def check_precond_dummy(solver, case):
 
     precond = LinearOperator(A.shape, identity, rmatvec=identity)
 
-    if solver is qmr:
-        x, info = solver(A, b, M1=precond, M2=precond, x0=x0, tol=tol)
+    if case.solver is qmr:
+        x, info = case.solver(A, b, M1=precond, M2=precond, x0=x0, rtol=rtol)
     else:
-        x, info = solver(A, b, M=precond, x0=x0, tol=tol)
+        x, info = case.solver(A, b, M=precond, x0=x0, rtol=rtol)
     assert info == 0
-    assert_normclose(A @ x, b, tol)
+    assert norm(A @ x - b) <= norm(b) * rtol
 
     A = aslinearoperator(A)
     A.psolve = identity
     A.rpsolve = identity
 
-    x, info = solver(A, b, x0=x0, tol=tol)
+    x, info = case.solver(A, b, x0=x0, rtol=rtol)
     assert info == 0
-    assert_normclose(A @ x, b, tol=tol)
+    assert norm(A @ x - b) <= norm(b) * rtol
 
 
-def test_precond_dummy(solver, case):
-    if solver in case.skip + case.nonconvergence:
-        pytest.skip("unsupported combination")
-    with suppress_warnings() as sup:
-        sup.filter(DeprecationWarning, ".*called without specifying.*")
-        check_precond_dummy(solver, case)
+# Specific test for poisson1d and poisson2d cases
+@pytest.mark.parametrize('case', [x for x in IterativeParams().cases
+                                  if x.name in ('poisson1d', 'poisson2d')],
+                         ids=['poisson1d', 'poisson2d'])
+def test_precond_inverse(case):
+    for solver in _SOLVERS:
+        if solver in case.skip or solver is qmr:
+            continue
 
+        rtol = 1e-8
 
-def check_precond_inverse(solver, case):
-    tol = 1e-8
+        def inverse(b, which=None):
+            """inverse preconditioner"""
+            A = case.A
+            if not isinstance(A, np.ndarray):
+                A = A.toarray()
+            return np.linalg.solve(A, b)
 
-    def inverse(b, which=None):
-        """inverse preconditioner"""
-        A = case.A
-        if not isinstance(A, np.ndarray):
-            A = A.toarray()
-        return np.linalg.solve(A, b)
+        def rinverse(b, which=None):
+            """inverse preconditioner"""
+            A = case.A
+            if not isinstance(A, np.ndarray):
+                A = A.toarray()
+            return np.linalg.solve(A.T, b)
 
-    def rinverse(b, which=None):
-        """inverse preconditioner"""
-        A = case.A
-        if not isinstance(A, np.ndarray):
-            A = A.toarray()
-        return np.linalg.solve(A.T, b)
+        matvec_count = [0]
 
-    matvec_count = [0]
+        def matvec(b):
+            matvec_count[0] += 1
+            return case.A @ b
 
-    def matvec(b):
-        matvec_count[0] += 1
-        return case.A @ b
+        def rmatvec(b):
+            matvec_count[0] += 1
+            return case.A.T @ b
 
-    def rmatvec(b):
-        matvec_count[0] += 1
-        return case.A.T @ b
+        b = case.b
+        x0 = 0 * b
 
-    b = case.b
-    x0 = 0 * b
+        A = LinearOperator(case.A.shape, matvec, rmatvec=rmatvec)
+        precond = LinearOperator(case.A.shape, inverse, rmatvec=rinverse)
 
-    A = LinearOperator(case.A.shape, matvec, rmatvec=rmatvec)
-    precond = LinearOperator(case.A.shape, inverse, rmatvec=rinverse)
+        # Solve with preconditioner
+        matvec_count = [0]
+        x, info = solver(A, b, M=precond, x0=x0, rtol=rtol)
 
-    # Solve with preconditioner
-    matvec_count = [0]
-    x, info = solver(A, b, M=precond, x0=x0, tol=tol)
-
-    assert info == 0
-    assert_normclose(case.A @ x, b, tol)
-
-    # Solution should be nearly instant
-    assert matvec_count[0] <= 3
-
-
-def test_precond_inverse(solver, case):
-    if (solver in case.skip or solver is qmr
-            or case.name not in ("poisson1d", "poisson2d")):
-        pytest.skip("unsupported combination")
-    with suppress_warnings() as sup:
-        sup.filter(DeprecationWarning, ".*called without specifying.*")
-        check_precond_inverse(solver, case)
-
-
-def test_reentrancy(solver):
-    reentrant = [lgmres, minres, gcrotmk, tfqmr]
-    with suppress_warnings() as sup:
-        sup.filter(DeprecationWarning, ".*called without specifying.*")
-        _check_reentrancy(solver, solver in reentrant)
-
-
-def _check_reentrancy(solver, is_reentrant):
-    def matvec(x):
-        A = np.array([[1.0, 0, 0], [0, 2.0, 0], [0, 0, 3.0]])
-        y, info = solver(A, x)
         assert info == 0
-        return y
-    b = np.array([1, 1. / 2, 1. / 3])
-    op = LinearOperator((3, 3), matvec=matvec, rmatvec=matvec,
-                        dtype=b.dtype)
+        assert norm(case.A @ x - b) <= norm(b) * rtol
 
-    if not is_reentrant:
-        pytest.raises(RuntimeError, solver, op, b)
-    else:
-        y, info = solver(op, b)
-        assert info == 0
-        assert_allclose(y, [1, 1, 1])
+        # Solution should be nearly instant
+        assert matvec_count[0] <= 3
 
 
 def test_atol(solver):
     # TODO: minres / tfqmr. It didn't historically use absolute tolerances, so
     # fixing it is less urgent.
     if solver in (minres, tfqmr):
-        pytest.skip("TODO")
+        pytest.skip("TODO: Add atol to minres/tfqmr")
+
+    # Historically this is tested as below, all pass but for some reason
+    # gcrotmk is over-sensitive to difference between random.seed/rng.random
+    # Hence tol lower bound is changed from -10 to -9
+    # np.random.seed(1234)
+    # A = np.random.rand(10, 10)
+    # A = A @ A.T + 10 * np.eye(10)
+    # b = 1e3*np.random.rand(10)
+
+    rng = np.random.default_rng(168441431005389)
+    A = rng.uniform(size=[10, 10])
+    A = A @ A.T + 10*np.eye(10)
+    b = 1e3 * rng.uniform(size=10)
 
-    np.random.seed(1234)
-    A = np.random.rand(10, 10)
-    A = A @ A.T + 10 * np.eye(10)
-    b = 1e3 * np.random.rand(10)
     b_norm = np.linalg.norm(b)
 
-    tols = np.r_[0, np.logspace(np.log10(1e-10), np.log10(1e2), 7), np.inf]
+    tols = np.r_[0, np.logspace(-9, 2, 7), np.inf]
 
     # Check effect of badly scaled preconditioners
-    M0 = np.random.randn(10, 10)
+    M0 = rng.standard_normal(size=(10, 10))
     M0 = M0 @ M0.T
     Ms = [None, 1e-6 * M0, 1e6 * M0]
 
-    for M, tol, atol in itertools.product(Ms, tols, tols):
-        if tol == 0 and atol == 0:
+    for M, rtol, atol in itertools.product(Ms, tols, tols):
+        if rtol == 0 and atol == 0:
             continue
 
         if solver is qmr:
@@ -408,58 +396,55 @@ def test_atol(solver):
                 M2 = aslinearoperator(np.eye(10))
             else:
                 M2 = None
-            x, info = solver(A, b, M1=M, M2=M2, tol=tol, atol=atol)
+            x, info = solver(A, b, M1=M, M2=M2, rtol=rtol, atol=atol)
         else:
-            x, info = solver(A, b, M=M, tol=tol, atol=atol)
+            x, info = solver(A, b, M=M, rtol=rtol, atol=atol)
 
         assert info == 0
         residual = A @ x - b
         err = np.linalg.norm(residual)
-        atol2 = tol * b_norm
+        atol2 = rtol * b_norm
         # Added 1.00025 fudge factor because of `err` exceeding `atol` just
         # very slightly on s390x (see gh-17839)
         assert err <= 1.00025 * max(atol, atol2)
 
 
 def test_zero_rhs(solver):
-    np.random.seed(1234)
-    A = np.random.rand(10, 10)
+    rng = np.random.default_rng(1684414984100503)
+    A = rng.random(size=[10, 10])
     A = A @ A.T + 10 * np.eye(10)
 
     b = np.zeros(10)
-    tols = np.r_[np.logspace(np.log10(1e-10), np.log10(1e2), 7)]
+    tols = np.r_[np.logspace(-10, 2, 7)]
 
     for tol in tols:
-        with suppress_warnings() as sup:
-            sup.filter(DeprecationWarning, ".*called without specifying.*")
+        x, info = solver(A, b, rtol=tol)
+        assert info == 0
+        assert_allclose(x, 0., atol=1e-15)
 
-            x, info = solver(A, b, tol=tol)
+        x, info = solver(A, b, rtol=tol, x0=ones(10))
+        assert info == 0
+        assert_allclose(x, 0., atol=tol)
+
+        if solver is not minres:
+            x, info = solver(A, b, rtol=tol, atol=0, x0=ones(10))
+            if info == 0:
+                assert_allclose(x, 0)
+
+            x, info = solver(A, b, rtol=tol, atol=tol)
             assert info == 0
-            assert_allclose(x, 0., atol=1e-15)
+            assert_allclose(x, 0, atol=1e-300)
 
-            x, info = solver(A, b, tol=tol, x0=ones(10))
+            x, info = solver(A, b, rtol=tol, atol=0)
             assert info == 0
-            assert_allclose(x, 0., atol=tol)
-
-            if solver is not minres:
-                x, info = solver(A, b, tol=tol, atol=0, x0=ones(10))
-                if info == 0:
-                    assert_allclose(x, 0)
-
-                x, info = solver(A, b, tol=tol, atol=tol)
-                assert info == 0
-                assert_allclose(x, 0, atol=1e-300)
-
-                x, info = solver(A, b, tol=tol, atol=0)
-                assert info == 0
-                assert_allclose(x, 0, atol=1e-300)
+            assert_allclose(x, 0, atol=1e-300)
 
 
 @pytest.mark.xfail(reason="see gh-18697")
 def test_maxiter_worsening(solver):
-    if solver not in (gmres, lgmres):
+    if solver not in (gmres, lgmres, qmr):
         # these were skipped from the very beginning, see gh-9201; gh-14160
-        pytest.skip("unsupported combination")
+        pytest.skip("Solver breakdown case")
     # Check error does not grow (boundlessly) with increasing maxiter.
     # This can occur due to the solvers hitting close to breakdown,
     # which they should detect and halt as necessary.
@@ -479,60 +464,65 @@ def test_maxiter_worsening(solver):
                   [0.1112795288033368, 0j, 0j, -0.16127952880333785]])
     v = np.ones(4)
     best_error = np.inf
-    tol = 7 if platform.machine() == 'aarch64' else 5
+
+    # Unable to match the Fortran code tolerance levels with this example
+    # Original tolerance values
+
+    # slack_tol = 7 if platform.machine() == 'aarch64' else 5
+    slack_tol = 9
 
     for maxiter in range(1, 20):
-        x, info = solver(A, v, maxiter=maxiter, tol=1e-8, atol=0)
+        x, info = solver(A, v, maxiter=maxiter, rtol=1e-8, atol=0)
 
         if info == 0:
-            assert np.linalg.norm(A @ x - v) <= 1e-8 * np.linalg.norm(v)
+            assert norm(A @ x - v) <= 1e-8 * norm(v)
 
         error = np.linalg.norm(A @ x - v)
         best_error = min(best_error, error)
 
         # Check with slack
-        assert error <= tol * best_error
+        assert error <= slack_tol * best_error
 
 
 def test_x0_working(solver):
     # Easy problem
-    np.random.seed(1)
+    rng = np.random.default_rng(1685363802304750)
     n = 10
-    A = np.random.rand(n, n)
+    A = rng.random(size=[n, n])
     A = A @ A.T
-    b = np.random.rand(n)
-    x0 = np.random.rand(n)
+    b = rng.random(n)
+    x0 = rng.random(n)
 
     if solver is minres:
-        kw = dict(tol=1e-6)
+        kw = dict(rtol=1e-6)
     else:
-        kw = dict(atol=0, tol=1e-6)
+        kw = dict(atol=0, rtol=1e-6)
 
     x, info = solver(A, b, **kw)
     assert info == 0
-    assert np.linalg.norm(A @ x - b) <= 1e-6 * np.linalg.norm(b)
+    assert norm(A @ x - b) <= 1e-6 * norm(b)
 
     x, info = solver(A, b, x0=x0, **kw)
     assert info == 0
-    assert np.linalg.norm(A @ x - b) <= 2e-6 * np.linalg.norm(b)
+    assert norm(A @ x - b) <= 2e-6*norm(b)
 
 
-def test_x0_equals_Mb(solver, case):
-    if solver in case.skip or solver is tfqmr:
-        pytest.skip("unsupported combination")
-    with suppress_warnings() as sup:
-        sup.filter(DeprecationWarning, ".*called without specifying.*")
-        A = case.A
-        b = case.b
-        x0 = 'Mb'
-        tol = 1e-8
-        x, info = solver(A, b, x0=x0, tol=tol)
+def test_x0_equals_Mb(case):
+    if case.solver is tfqmr:
+        pytest.skip("Solver does not support x0='Mb'")
+    A = case.A
+    b = case.b
+    x0 = 'Mb'
+    rtol = 1e-8
+    x, info = case.solver(A, b, x0=x0, rtol=rtol)
 
-        assert_array_equal(x0, 'Mb')  # ensure that x0 is not overwritten
-        assert info == 0
-        assert_normclose(A @ x, b, tol=tol)
+    assert_array_equal(x0, 'Mb')  # ensure that x0 is not overwritten
+    assert info == 0
+    assert norm(A @ x - b) <= rtol * norm(b)
 
 
+# Specific tfqmr test
+@pytest.mark.parametrize('case', IterativeParams().cases)
 def test_show(case, capsys):
     def cb(x):
         pass
@@ -553,9 +543,24 @@ def test_show(case, capsys):
     assert err == ""
 
 
-# -----------------------------------------------------------------------------
+def test_positional_deprecation(solver):
+    # from test_x0_working
+    rng = np.random.default_rng(1685363802304750)
+    n = 10
+    A = rng.random(size=[n, n])
+    A = A @ A.T
+    b = rng.random(n)
+    x0 = rng.random(n)
+    with pytest.deprecated_call(
+        # due to the use of the _deprecate_positional_args decorator, it's not possible
+        # to separate the two warnings (1 for positional use, 1 for `tol` deprecation).
+        match="use keyword arguments.*|argument `tol` is deprecated.*"
+    ):
+        solver(A, b, x0, 1e-5)
+
 
 class TestQMR:
+    @pytest.mark.filterwarnings('ignore::scipy.sparse.SparseEfficiencyWarning')
     def test_leftright_precond(self):
         """Check that QMR works with left and right preconditioners"""
 
@@ -570,12 +575,8 @@ class TestQMR:
 
         L = spdiags([-dat / 2, dat], [-1, 0], n, n)
         U = spdiags([4 * dat, -dat], [0, 1], n, n)
-
-        with suppress_warnings() as sup:
-            sup.filter(SparseEfficiencyWarning,
-                       "splu converted its input to CSC format")
-            L_solver = splu(L)
-            U_solver = splu(U)
+        L_solver = splu(L)
+        U_solver = splu(U)
 
         def L_solve(b):
             return L_solver.solve(b)
@@ -592,12 +593,11 @@ class TestQMR:
         M1 = LinearOperator((n, n), matvec=L_solve, rmatvec=LT_solve)
         M2 = LinearOperator((n, n), matvec=U_solve, rmatvec=UT_solve)
 
-        with suppress_warnings() as sup:
-            sup.filter(DeprecationWarning, ".*called without specifying.*")
-            x, info = qmr(A, b, tol=1e-8, maxiter=15, M1=M1, M2=M2)
+        rtol = 1e-8
+        x, info = qmr(A, b, rtol=rtol, maxiter=15, M1=M1, M2=M2)
 
         assert info == 0
-        assert_normclose(A @ x, b, tol=1e-8)
+        assert norm(A @ x - b) <= rtol * norm(b)
 
 
 class TestGMRES:
@@ -606,9 +606,7 @@ class TestGMRES:
         b = np.zeros(10)
         b[0] = 1
 
-        with suppress_warnings() as sup:
-            sup.filter(DeprecationWarning, ".*called without specifying.*")
-            x_gm, err = gmres(A, b, restart=5, maxiter=1)
+        x_gm, err = gmres(A, b, restart=5, maxiter=1)
 
         assert_allclose(x_gm[0], 0.359, rtol=1e-2)
 
@@ -632,10 +630,8 @@ class TestGMRES:
         def callback(r):
             return store_residual(r, rvec)
 
-        with suppress_warnings() as sup:
-            sup.filter(DeprecationWarning, ".*called without specifying.*")
-            x, flag = gmres(A, b, x0=zeros(A.shape[0]), tol=1e-16,
-                            maxiter=maxiter, callback=callback)
+        x, flag = gmres(A, b, x0=zeros(A.shape[0]), rtol=1e-16,
+                        maxiter=maxiter, callback=callback)
 
         # Expected output from SciPy 1.0.0
         assert_allclose(rvec, array([1.0, 0.81649658092772603]), rtol=1e-10)
@@ -644,10 +640,8 @@ class TestGMRES:
         M = 1e-3 * np.eye(A.shape[0])
         rvec = zeros(maxiter + 1)
         rvec[0] = 1.0
-        with suppress_warnings() as sup:
-            sup.filter(DeprecationWarning, ".*called without specifying.*")
-            x, flag = gmres(A, b, M=M, tol=1e-16, maxiter=maxiter,
-                            callback=callback)
+        x, flag = gmres(A, b, M=M, rtol=1e-16, maxiter=maxiter,
+                        callback=callback)
 
         # Expected output from SciPy 1.0.0
         # (callback has preconditioned residual!)
@@ -658,43 +652,31 @@ class TestGMRES:
         # Check we don't segfault on gmres with complex argument
         A = eye(2)
         b = ones(2)
-        with suppress_warnings() as sup:
-            sup.filter(DeprecationWarning, ".*called without specifying.*")
-            r_x, r_info = gmres(A, b)
-            r_x = r_x.astype(complex)
-
-            x, info = gmres(A.astype(complex), b.astype(complex))
+        r_x, r_info = gmres(A, b)
+        r_x = r_x.astype(complex)
+        x, info = gmres(A.astype(complex), b.astype(complex))
 
         assert iscomplexobj(x)
         assert_allclose(r_x, x)
         assert r_info == info
 
     def test_atol_legacy(self):
-        with suppress_warnings() as sup:
-            sup.filter(DeprecationWarning, ".*called without specifying.*")
 
-            # Check the strange legacy behavior: the tolerance is interpreted
-            # as atol, but only for the initial residual
-            A = eye(2)
-            b = 1e-6 * ones(2)
-            x, info = gmres(A, b, tol=1e-5)
-            assert_array_equal(x, np.zeros(2))
+        A = eye(2)
+        b = ones(2)
+        x, info = gmres(A, b, rtol=1e-5)
+        assert np.linalg.norm(A @ x - b) <= 1e-5 * np.linalg.norm(b)
+        assert_allclose(x, b, atol=0, rtol=1e-8)
 
-            A = eye(2)
-            b = ones(2)
-            x, info = gmres(A, b, tol=1e-5)
-            assert np.linalg.norm(A @ x - b) <= 1e-5 * np.linalg.norm(b)
-            assert_allclose(x, b, atol=0, rtol=1e-8)
-
-            rndm = np.random.RandomState(12345)
-            A = rndm.rand(30, 30)
-            b = 1e-6 * ones(30)
-            x, info = gmres(A, b, tol=1e-7, restart=20)
-            assert np.linalg.norm(A @ x - b) > 1e-7
+        rndm = np.random.RandomState(12345)
+        A = rndm.rand(30, 30)
+        b = 1e-6 * ones(30)
+        x, info = gmres(A, b, rtol=1e-7, restart=20)
+        assert np.linalg.norm(A @ x - b) > 1e-7
 
         A = eye(2)
         b = 1e-10 * ones(2)
-        x, info = gmres(A, b, tol=1e-8, atol=0)
+        x, info = gmres(A, b, rtol=1e-8, atol=0)
         assert np.linalg.norm(A @ x - b) <= 1e-8 * np.linalg.norm(b)
 
     def test_defective_precond_breakdown(self):
@@ -706,7 +688,7 @@ class TestGMRES:
         x = np.array([1, 0, 0])
         A = np.diag([2, 3, 4])
 
-        x, info = gmres(A, b, x0=x, M=M, tol=1e-15, atol=0)
+        x, info = gmres(A, b, x0=x, M=M, rtol=1e-15, atol=0)
 
         # Should not return nans, nor terminate with false success
         assert not np.isnan(x).any()
@@ -720,12 +702,13 @@ class TestGMRES:
         # Breakdown due to defective matrix
         A = np.array([[0, 1, 0], [1, 0, 0], [0, 0, 0]])
         b = np.array([1, 0, 1])
-        x, info = gmres(A, b, tol=1e-8, atol=0)
+        rtol = 1e-8
+        x, info = gmres(A, b, rtol=rtol, atol=0)
 
         # Should not return nans, nor terminate with false success
         assert not np.isnan(x).any()
         if info == 0:
-            assert np.linalg.norm(A @ x - b) <= 1e-8 * np.linalg.norm(b)
+            assert np.linalg.norm(A @ x - b) <= rtol * np.linalg.norm(b)
 
         # The solution should be OK outside null space of A
         assert_allclose(A @ (A @ x), A @ b)
@@ -746,35 +729,33 @@ class TestGMRES:
             cb_count[0] += 1
             assert isinstance(x, np.ndarray)
 
-        with suppress_warnings() as sup:
-            sup.filter(DeprecationWarning, ".*called without specifying.*")
-            # 2 iterations is not enough to solve the problem
-            cb_count = [0]
-            x, info = gmres(A, b, tol=1e-6, atol=0, callback=pr_norm_cb, maxiter=2,
-                            restart=50)
-            assert info == 2
-            assert cb_count[0] == 2
+        # 2 iterations is not enough to solve the problem
+        cb_count = [0]
+        x, info = gmres(A, b, rtol=1e-6, atol=0, callback=pr_norm_cb,
+                        maxiter=2, restart=50)
+        assert info == 2
+        assert cb_count[0] == 2
 
         # With `callback_type` specified, no warning should be raised
         cb_count = [0]
-        x, info = gmres(A, b, tol=1e-6, atol=0, callback=pr_norm_cb,
+        x, info = gmres(A, b, rtol=1e-6, atol=0, callback=pr_norm_cb,
                         maxiter=2, restart=50, callback_type='legacy')
         assert info == 2
         assert cb_count[0] == 2
 
         # 2 restart cycles is enough to solve the problem
         cb_count = [0]
-        x, info = gmres(A, b, tol=1e-6, atol=0, callback=pr_norm_cb,
+        x, info = gmres(A, b, rtol=1e-6, atol=0, callback=pr_norm_cb,
                         maxiter=2, restart=50, callback_type='pr_norm')
         assert info == 0
         assert cb_count[0] > 2
 
         # 2 restart cycles is enough to solve the problem
         cb_count = [0]
-        x, info = gmres(A, b, tol=1e-6, atol=0, callback=x_cb, maxiter=2,
+        x, info = gmres(A, b, rtol=1e-6, atol=0, callback=x_cb, maxiter=2,
                         restart=50, callback_type='x')
         assert info == 0
-        assert cb_count[0] == 2
+        assert cb_count[0] == 1
 
     def test_callback_x_monotonic(self):
         # Check that callback_type='x' gives monotonic norm decrease
@@ -791,11 +772,10 @@ class TestGMRES:
             prev_r[0] = r
             count[0] += 1
 
-        x, info = gmres(A, b, tol=1e-6, atol=0, callback=x_cb, maxiter=20, restart=10,
-                        callback_type='x')
+        x, info = gmres(A, b, rtol=1e-6, atol=0, callback=x_cb, maxiter=20,
+                        restart=10, callback_type='x')
         assert info == 20
-        assert count[0] == 21
-        x_cb(x)
+        assert count[0] == 20
 
     def test_restrt_dep(self):
         with pytest.warns(
