@@ -96,10 +96,9 @@
  */
 
 #include "mconf.h"
-double gdtri(double, double, double);
 
-double gdtr(a, b, x)
-double a, b, x;
+
+double gdtr(double a, double b, double x)
 {
 
     if (x < 0.0) {
@@ -110,8 +109,7 @@ double a, b, x;
 }
 
 
-double gdtrc(a, b, x)
-double a, b, x;
+double gdtrc(double a, double b, double x)
 {
 
     if (x < 0.0) {
@@ -122,8 +120,7 @@ double a, b, x;
 }
 
 
-double gdtri(a, b, y)
-double a, b, y;
+double gdtri(double a, double b, double y)
 {
 
     if ((y < 0.0) || (y > 1.0) || (a <= 0.0) || (b < 0.0)) {
